import type React from 'react';
import styles from './App.module.css';
import Layout from '@/features/layout/Layout';
import clsx from 'clsx';
import { useAppSelector } from '@/app/hooks';
import { Outlet } from 'react-router-dom';
import { selectContextualPanelOpen } from '@/features/ui/uiSlice';
import Canvas from '@/features/canvas/Canvas';
import ErrorBoundary from '@/components/error/ErrorBoundary';
import PrimaryMenu from '@/components/sidebar/PrimaryMenu';
import Topbar from '@/components/topbar/Topbar';
import useSyncComponentId from '@/hooks/useSyncComponentId';
import ZoomControl from '@/components/zoom/ZoomControl';

const App: React.FC = () => {
  const contextualPanelOpen = useAppSelector(selectContextualPanelOpen);

  // Hook to keep the selected component ID in state in sync with :componentId in the url params.
  useSyncComponentId();

  return (
    <ErrorBoundary variant="page">
      <div
        className={clsx(styles.app, {
          [styles.rightSideBarOpen]: contextualPanelOpen,
        })}
      >
        <Canvas />
        <ErrorBoundary
          variant="alert"
          title="An unexpected error has occurred while fetching layouts."
        >
          <Layout />
        </ErrorBoundary>
        <Topbar />
        <PrimaryMenu />
        <Outlet />
        <ZoomControl />
        <div id="menuBarContainer" className="menuBarContainer"></div>
      </div>
    </ErrorBoundary>
  );
};

export default App;
